package dev.ragnarok.fenrir.model

import androidx.annotation.IntDef

@IntDef(
    PhotoSize.S,
    PhotoSize.M,
    PhotoSize.X,
    PhotoSize.Y,
    PhotoSize.Z,
    PhotoSize.W,
    PhotoSize.O,
    PhotoSize.P,
    PhotoSize.Q,
    PhotoSize.R
)
@Retention(
    AnnotationRetention.SOURCE
)
annotation class PhotoSize {
    companion object {
        /**
         * пропорциональная копия изображения с максимальной шириной 75px
         */
        const val S = 1

        /**
         * пропорциональная копия изображения с максимальной шириной 130px
         */
        const val M = 2

        /**
         * пропорциональная копия изображения с максимальной шириной 604px
         */
        const val X = 3

        /**
         * пропорциональная копия изображения с максимальной стороной 807px
         */
        const val Y = 4

        /**
         * пропорциональная копия изображения с максимальным размером 1280x1024
         */
        const val Z = 5

        /**
         * пропорциональная копия изображения с максимальным размером 2560x2048px
         */
        const val W = 6

        /**
         * если соотношение "ширина/высота" исходного изображения меньше или равно 3:2,
         * то пропорциональная копия с максимальной шириной 130px.
         * Если соотношение "ширина/высота" больше 3:2, то копия обрезанного слева изображения
         * с максимальной шириной 130px и соотношением сторон 3:2
         */
        const val O = 7

        /**
         * если соотношение "ширина/высота" исходного изображения меньше или равно 3:2,
         * то пропорциональная копия с максимальной шириной 200px.
         * Если соотношение "ширина/высота" больше 3:2, то копия обрезанного слева и
         * справа изображения с максимальной шириной 200px и соотношением сторон 3:2
         */
        const val P = 8

        /**
         * если соотношение "ширина/высота" исходного изображения меньше или равно 3:2,
         * то пропорциональная копия с максимальной шириной 320px.
         * Если соотношение "ширина/высота" больше 3:2, то копия обрезанного слева и
         * справа изображения с максимальной шириной 320px и соотношением сторон 3:2
         */
        const val Q = 9

        /**
         * если соотношение "ширина/высота" исходного изображения меньше или равно 3:2,
         * то пропорциональная копия с максимальной шириной 510px.
         * Если соотношение "ширина/высота" больше 3:2, то копия обрезанного слева
         * и справа изображения с максимальной шириной 510px и соотношением сторон 3:2
         */
        const val R = 10
    }
}